PRINT N'Altering Procedure [dbo].[GetProcessFiltersData]...';


GO
ALTER PROCEDURE [dbo].[GetProcessFiltersData] ( 
	@VCC VARCHAR(50) = NULL, 
	@ProcessId VARCHAR(50) = NULL,
	@EvaluationFormatId VARCHAR(50) = NULL
)
AS
BEGIN
	-- ELIMINAMOS LA TABLA TEMPORAL EN CASO EXISTA
	DROP TABLE IF EXISTS  #ProcessFiltersData 

	-- CREAMOS LA TABLA TEMPORAL PARA LOS RESUMENES
	CREATE TABLE #ProcessFiltersData (
		[Channel] VARCHAR(50),
		[Campaign] VARCHAR (50),
		[AgentId] VARCHAR(50),
		[AgentName] VARCHAR(100),
		[DispositionCode] VARCHAR(50),
		[Category1] VARCHAR(100),
		[Category2] VARCHAR(100),
		[Category3] VARCHAR(100),
		[Description] VARCHAR(100),
		[StartDate] DATETIME,
		[EndDate] DATETIME,
		[DurationTime] NUMERIC(12,3)
	)
	
	-- CONSULTAMOS DATA DE LA TABLA
	--SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = 'AsyncInteraction' AND COLUMN_NAME IN ('Agent', 'AgentName', 'FirstTakedAgent', 'FirstTakedAgentName', 'LastTakedAgent', 'LastTakedAgentName')
	--SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = 'Audio' AND COLUMN_NAME IN ('Category1', 'Category2', 'Category3', 'Description', 'DispositionCode')
	
	-- INSERTAMOS LA DATA DE AUDIOS
	IF @EvaluationFormatId IS NULL
	BEGIN
		INSERT INTO #ProcessFiltersData
			SELECT 'CALL',
					A.[Campaign],
					A.[Agent],
					A.[AgentName],
					A.[DispositionCode],
					A.[Category1],
					A.[Category2],
					A.[Category3],
					A.[Description],
					A.[StartDate],
					A.[EndDate],
					A.[DurationLinguisticTime]

				FROM [dbo].[ProcessAudio] PA (NOLOCK) 
				INNER JOIN [Audio] A (NOLOCK)
					ON A.[VCC] = PA.[VCC]
						AND A.[ID] = PA.[AudioId]
						AND A.[Section] = PA.[Section]
						AND A.[Part] = PA.[Part]
				WHERE PA.[VCC] = @VCC
					AND PA.[ProcessId] = @ProcessId
	END
	ELSE
	BEGIN
		INSERT INTO #ProcessFiltersData
			SELECT 'CALL',
					A.[Campaign],
					A.[Agent],
					A.[AgentName],
					A.[DispositionCode],
					A.[Category1],
					A.[Category2],
					A.[Category3],
					A.[Description],
					A.[StartDate],
					A.[EndDate],
					A.[DurationLinguisticTime]
				FROM [dbo].[ProcessAudio] PA (NOLOCK) 
				INNER JOIN [Audio] A (NOLOCK)
					ON A.[VCC] = PA.[VCC]
						AND A.[ID] = PA.[AudioId]
						AND A.[Section] = PA.[Section]
						AND A.[Part] = PA.[Part]
				INNER JOIN [dbo].[CampaignEvaluationFormat] CEF 
					ON A.[VCC] = CEF.[Instance]
						AND A.[Campaign] = CEF.[IdCampaign]
				WHERE PA.[VCC] = @VCC
					AND PA.[ProcessId] = @ProcessId
					AND CEF.[IdEvaluationFormat] = @EvaluationFormatId
		
	END
	
	-- INSERTAMOS LA DATA DE TEXTOS
	IF @EvaluationFormatId IS NULL
	BEGIN
		INSERT INTO #ProcessFiltersData
			SELECT AI.[Channel],
					AI.[Campaign],
					ISNULL(AI.[LastTakedAgent], AI.[FirstTakedAgent]),
					ISNULL(AI.[LastTakedAgentName], AI.[FirstTakedAgentName]),
					AI.[DispositionCode],
					NULL,--AI.[Category1],
					NULL,--AI.[Category2],
					NULL,--AI.[Category3],
					NULL,--AI.[Description],
					AI.[StartDate],
					AI.[EndDate],
					AI.[DurationTime]
				FROM [dbo].[ProcessAsyncInteraction] PAI (NOLOCK) 
				INNER JOIN [AsyncInteraction] AI (NOLOCK)
					ON AI.[VCC] = PAI.[VCC] 
						AND AI.[ID] = PAI.[InteractionId]
						AND AI.[Folder] = PAI.[FormattedEndDate]
						AND AI.[Campaign] = PAI.[Campaign]
				WHERE PAI.[VCC] = @VCC
					AND PAI.[ProcessId] = @ProcessId
					
	END
	ELSE
	BEGIN
		INSERT INTO #ProcessFiltersData
			SELECT AI.[Channel],
					AI.[Campaign],
					ISNULL(AI.[LastTakedAgent], AI.[FirstTakedAgent]),
					ISNULL(AI.[LastTakedAgentName], AI.[FirstTakedAgentName]),
					AI.[DispositionCode],
					NULL,--AI.[Category1],
					NULL,--AI.[Category2],
					NULL,--AI.[Category3],
					NULL,--AI.[Description]
					AI.[StartDate],
					AI.[EndDate],
					AI.[DurationTime]
				FROM [dbo].[ProcessAsyncInteraction] PAI (NOLOCK) 
				INNER JOIN [AsyncInteraction] AI (NOLOCK)
					ON AI.[VCC] = PAI.[VCC] 
						AND AI.[ID] = PAI.[InteractionId]
						AND AI.[Folder] = PAI.[FormattedEndDate]
						AND AI.[Campaign] = PAI.[Campaign]
				INNER JOIN [dbo].[CampaignEvaluationFormat] CEF 
					ON AI.[VCC] = CEF.[Instance]
						AND AI.[Campaign] = CEF.[IdCampaign]
				WHERE PAI.[VCC] = @VCC
					AND PAI.[ProcessId] = @ProcessId
					AND CEF.[IdEvaluationFormat] = @EvaluationFormatId
				
		
	END

	-- SELECCIONAMOS AGRUPADO POR FILTRO
	SELECT [Channel] 'channel' 
		FROM #ProcessFiltersData
		WHERE [Channel] IS NOT NULL 
		GROUP BY [Channel] ORDER BY [Channel] ASC

	SELECT [Campaign] 'campaignId'	
		FROM #ProcessFiltersData 
		WHERE [Campaign] IS NOT NULL 
		GROUP BY [Campaign] ORDER BY [Campaign] ASC

	SELECT [AgentId] 'agentId', 
			[AgentName] 'agentName'
		FROM #ProcessFiltersData 
		WHERE [AgentId] IS NOT NULL
		GROUP BY [AgentId], [AgentName] 
		ORDER BY [AgentId], [AgentName] ASC

	SELECT [DispositionCode]	'dispositionCode',
			[Category1]			'description',
			[Category2]			'description2',
			[Category3]			'description3',
			[Description]		'description4'
		FROM #ProcessFiltersData 
		WHERE  [DispositionCode] IS NOT NULL
		GROUP BY [DispositionCode], [Category1], [Category2], [Category3], [Description] 
		--ORDER BY [DispositionCode], [Category1], [Category2], [Category3], [Description]  ASC
	SELECT MIN(StartDate) 'startDate',MAX(EndDate) 'endDate' 
	FROM #ProcessFiltersData
	SELECT [Channel] 'interactionType', COUNT(1) 'totalInteractions' ,COUNT(1) 'totalInteractionDetails',SUM([DurationTime]) 'totalInteractionTime'
	FROM #ProcessFiltersData
	GROUP BY [Channel]
	-- ELIMINAMOS LA TABLA TEMPORAL
	DROP TABLE #ProcessFiltersData 


END
GO
PRINT N'Altering Procedure [dbo].[GetProcesses]...'
GO
ALTER PROCEDURE [dbo].[GetProcesses]
(
	@VCC						VARCHAR(50) = NULL,
	@SearchCriteria				VARCHAR(50) = NULL,
	@SearchExpression			VARCHAR(50) = NULL,
	@StartDate					DATETIME,
	@EndDate					DATETIME,
	@Type						VARCHAR(50)
)
AS
BEGIN
	DECLARE @SQL NVARCHAR(MAX)
	IF @Type = 'ALL'
	BEGIN 
		SELECT [name],[name] as 'description'
			FROM dbo.[Process]
			WHERE [Status] = 'COMPLETE'
				AND [Type] = 'SPEECH'
				AND [VCC] = @VCC
		UNION ALL
		--UNIMOS A LOS PROCESOS DE QUALITY
		SELECT QPEP.[processId], EP.[Id]
		FROM dbo.[EvaluationPlan] (NOLOCK) EP 
			INNER JOIN (
			  SELECT [processId], MAX([CreatedDate]) AS CreatedDate
				FROM dbo.[EvaluationPlan] (NOLOCK)
				WHERE [Instance] =  @VCC
					AND [ProcessId] IS NOT NULL
					AND [Status] = 7
				GROUP BY [ProcessId]
			  ) AS QPEP
			ON EP.[CreatedDate] = QPEP.[CreatedDate]
				AND EP.[Instance] =  @VCC
				AND EP.[ProcessId] IS NOT NULL
				AND EP.[Status] =7


	END
	ELSE
	BEGIN
		SET @SQL = '
		SET NOCOUNT ON

		SELECT [VCC]																				''vcc'',
				[Name]																				''name'',
				[Description]																		''description'',
				[ClientId]																			''clientId'',
				[Percentage]																		''percentage'',
				[StatusDescription]																	''statusDescription'',
				(CASE WHEN [Status] IS NULL THEN ''UNKNOWN'' ELSE [Status] END)						''status'',
				[StartDate]																			''startDate'',
				[EndDate]																			''endDate'',
				[MinDurationTime]																	''minDurationTime'',
				[MaxDurationTime]																	''maxDurationTime'',
				[Language]																			''language'',
				[Environment]																		''environment'',
				[BatchScheduleId]																	''batchScheduleId'',
				[CreatedDate]																		''createdDate'',
				[CreatedByUserId]																	''createdByUserId'',
				[LastModifiedDate]																	''lastModifiedDate'',
				[LastModifiedByUserId]																''lastModifiedByUserId''
			FROM [Process] (NOLOCK)
			WHERE [VCC] = ''' + @VCC + '''
				AND ISNULL([Name], '''') LIKE CASE WHEN ''' + @SearchCriteria + ''' = ''name'' THEN ''%' + @SearchExpression + '%'' ELSE ''%'' END
				AND ISNULL([Description], '''') LIKE CASE WHEN ''' + @SearchCriteria + ''' = ''description'' THEN ''%' + @SearchExpression + '%'' ELSE ''%'' END
				AND ISNULL([ClientId], '''') LIKE CASE WHEN ''' + @SearchCriteria + ''' = ''clientId'' THEN ''%' + @SearchExpression + '%'' ELSE ''%'' END
				AND ISNULL([Status], '''') LIKE CASE WHEN ''' + @SearchCriteria + ''' = ''status'' THEN ''%' + @SearchExpression + '%'' ELSE ''%'' END
				AND ISNULL([Environment], '''') LIKE CASE WHEN ''' + @SearchCriteria + ''' = ''environment'' THEN ''%' + @SearchExpression + '%'' ELSE ''%'' END'

		IF(@StartDate IS NOT NULL AND @EndDate IS NOT NULL)
		BEGIN
			SET @SQL += '
				AND [CreatedDate] BETWEEN ''' + CONVERT(VARCHAR(10), @StartDate, 120) + ''' AND ''' + CONVERT(VARCHAR(10), @EndDate, 120) + '''
			'
		END
		SET @SQL += '
				AND [Type] = ''' + @Type + '''
		ORDER BY [Status] desc, [CreatedDate] desc 

		SET NOCOUNT OFF
		'
		EXEC(@SQL)
	END
END
GO
PRINT N'Update complete.';


GO
