-- ARCHIVO CON LAS ACTUALIZACIONES DE DATA DE SQL PARA MIGRAR AL NUEVO BUILD DEL NOMBRE DE LA CARPETA
-- DECLARAMOS DOS VARIABLES, UNA CON LA VERSION DE LA QUE PARTIMOS Y OTRA CON LA NUEVA VERSION QUE ESTAMOS APUNTANDO
DECLARE @OLD_DATABASE_BUILD_VERSION VARCHAR(50) = '1.7.0'
DECLARE @NEW_DATABASE_BUILD_VERSION VARCHAR(50) = '1.8.0'

-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
BEGIN TRY
    -- VALIDAMOS QUE LA VERSION EXISTENTE COINCIDE CON NUESTRA NEW_DATABASE_BUILD_VERSION
    IF NOT EXISTS (
        SELECT 1 
            FROM [dbo].[Version] V 
            WHERE V.[BuildVersion] = @OLD_DATABASE_BUILD_VERSION
    )	
        -- LANZAMOS UN MENSAJE DE ERROR INDICANDO QUE LA VERSION ACTUAL NO COINCIDE CON LA OLD_DATABASE_BUILD_VERSION
        RAISERROR('DATABASE_ERROR_INCORRECT_TARGET_VERSION', 16, 16)
        
    -- INICIAMOS LA TRANSACCION
    BEGIN TRANSACTION





    -- AQUI EMPIEZA LA SECCION CON LOS INSERT/UPDATE/DELETE PARA LA NUEVA VERSION


             --GUARDAR DATA HISTORICA EXISTENTE 

            INSERT INTO [dbo].[HistoricEvaluationFormat] 
            SELECT 
            EF.[Instance],
            EF.[ID],
            EF.[Description],
            EF.[type],
            EF.[EnvironmentType],
            EF.[MinAutomaticQualification],
            EF.[MinQualificationAcceptable],
            EF.[ManagementFormURL],
            EF.[Enabled],
            EF.[templateId]
            FROM [dbo].[EvaluationFormat] EF (NOLOCK) 
            JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
            ON EF.[Instance] = EP.[Instance] 
            AND EF.[ID] = EP.[IdEvaluationFormat]
            GROUP BY EF.[Instance],
            EF.[ID],
            EF.[Description],
            EF.[type],
            EF.[EnvironmentType],
            EF.[MinAutomaticQualification],
            EF.[MinQualificationAcceptable],
            EF.[ManagementFormURL],
            EF.[Enabled],
            EF.[templateId]


            INSERT INTO [dbo].[HistoricGroup] 
            SELECT 
                G.[Instance] ,
                G.[ID],
                G.[Description] ,
                G.[Enabled] 
            FROM [dbo].[Group] G (NOLOCK)
            JOIN [dbo].[EvaluationResult] ER (NOLOCK)
            ON ER.[Instance] = G.[Instance] AND ER.[IdGroup] = G.[ID]
            GROUP BY 
            G.[Instance] ,
            G.[ID],
            G.[Description] ,
            G.[Enabled]

            INSERT INTO [dbo].[HistoricSubGroup] 
            SELECT 
                        SG.[Instance] ,
                        SG.[ID],
                        SG.[Description] ,
                        SG.[Enabled]
            FROM [dbo].[SubGroup] SG (NOLOCK)
            JOIN [dbo].[EvaluationResult] ER (NOLOCK)
                        ON ER.[Instance] = SG.[Instance] AND ER.[IdSubGroup] = SG.[ID]
                        GROUP BY 
                        SG.[Instance] ,
                        SG.[ID],
                        SG.[Description] ,
                        SG.[Enabled]


            INSERT INTO [dbo].[HistoricAbility]
            SELECT 
                A.[Instance],
                A.[ID],
                A.[Title],
                A.[Detail],
                A.[Type],
                A.[Binary],
                A.[NotApply],
                A.[Distribute],
                A.[AcceptComments],
                A.[Enabled],
                A.[AnalysisType],
                A.[Query],
                A.[QueryName],
                A.[Language],
                A.[IsNegative]
                FROM [dbo].[Ability] A (NOLOCK)
                JOIN [dbo].[EvaluationResult] ER (NOLOCK)
                ON ER.[Instance] = A.[Instance] AND ER.[IdAbility] = A.[ID]
                GROUP BY 
                A.[Instance],
                A.[ID],
                A.[Title],
                A.[Detail],
                A.[Type],
                A.[Binary],
                A.[NotApply],
                A.[Distribute],
                A.[AcceptComments],
                A.[Enabled],
                A.[AnalysisType],
                A.[Query],
                A.[QueryName],
                A.[Language],
                A.[IsNegative]



            INSERT INTO [dbo].[HistoricAbilityEvaluationFormat]
            SELECT
                A.[Instance],
                A.[ID] ,
                A.[IdEvaluationFormat],
                A.[IdAbility],
                A.[IdGroup],
                A.[IdSubGroup],
                A.[Penalizes],
                A.[Score],
                A.[Order],
                A.[Conditional]
                FROM  [dbo].[AbilityEvaluationFormat] A (NOLOCK)
                JOIN [dbo].[EvaluationFormat] EF (NOLOCK)
                ON EF.[Instance] = A.[Instance] AND EF.[ID] = A.[IdEvaluationFormat]
                JOIN [dbo].[EvaluationPlan] EP (NOLOCK) 
                ON EP.[Instance] = EF.[Instance] AND EP.[IdEvaluationFormat] = EF.[ID]
                GROUP BY 
                A.[Instance],
                A.[ID] ,
                A.[IdEvaluationFormat],
                A.[IdAbility],
                A.[IdGroup],
                A.[IdSubGroup],
                A.[Penalizes],
                A.[Score],
                A.[Order],
                A.[Conditional]



            INSERT INTO [dbo].[HistoricAbilityQuery]
            SELECT 
                AQ.[ID],
                AQ.[VCC],
                AQ.[AbilityId],
                AQ.[QueryId],
                AQ.[QueryContent],
                AQ.[AcceptancePercentage],
                AQ.[Order]
            FROM [dbo].[AbilityQuery] AQ (NOLOCK)
            JOIN [dbo].[AbilityEvaluationFormat] AEF (NOLOCK)
            ON  AQ.[VCC] = AEF.[Instance] AND AEF.[IdAbility] = AQ.[AbilityId]
			JOIN [dbo].[EvaluationPlan] EP (NOLOCK) 
                ON EP.[Instance] = AEF.[Instance] AND EP.[IdEvaluationFormat] = AEF.[IdEvaluationFormat]
            GROUP BY 
            AQ.[ID],
                AQ.[VCC],
                AQ.[AbilityId],
                AQ.[QueryId],
                AQ.[QueryContent],
                AQ.[AcceptancePercentage],
                AQ.[Order]


            
            INSERT INTO [dbo].[HistoricAbilityQueryEntity]
            SELECT 
                AQE.[VCC],
                AQE.[AbilityQueryId],
                AQE.[EntityId],
                AQE.[EntityName],
                AQE.[AcceptancePercentage],
                AQE.[Order]
                FROM [dbo].[AbilityQueryEntity] AQE (NOLOCK)
                JOIN [dbo].[AbilityQuery] AQ (NOLOCK)
                ON AQ.[VCC] = AQE.[VCC] AND AQE.[AbilityQueryId] = AQ.[ID]
                JOIN [dbo].[AbilityEvaluationFormat] AEF (NOLOCK)
                ON  AQ.[VCC] = AEF.[Instance] AND AEF.[IdAbility] = AQ.[AbilityId]
                JOIN [dbo].[EvaluationPlan] EP (NOLOCK) 
                ON EP.[Instance] = AEF.[Instance] AND EP.[IdEvaluationFormat] = AEF.[IdEvaluationFormat]
                GROUP BY 
                AQE.[VCC],
                AQE.[AbilityQueryId],
                AQE.[EntityId],
                AQE.[EntityName],
                AQE.[AcceptancePercentage],
                AQE.[Order]
               

                INSERT INTO [dbo].[HistoricEvaluationFormatCustomField]
                SELECT 
                ECF.[VCC],
                ECF.[ID],
                ECF.[EvaluationFormatId],
                ECF.[CustomFieldId],
                ECF.[Order]
                FROM [dbo].[EvaluationFormatCustomField] ECF (NOLOCK)
                JOIN [dbo].[EvaluationFormat] EF (NOLOCK) 
                ON ECF.[VCC] = EF.[Instance] AND ECF.[EvaluationFormatId] = EF.[ID]
                JOIN [dbo].[EvaluationPlan] EP (NOLOCK) 
                ON EP.[Instance] = EF.[Instance] AND EP.[IdEvaluationFormat] = EF.[ID]
                GROUP BY
                ECF.[VCC],
                ECF.[ID],
                ECF.[EvaluationFormatId],
                ECF.[CustomFieldId],
                ECF.[Order]

            INSERT INTO [dbo].[HistoricCustomField]
            SELECT 
                C.[VCC],
                C.[ID],
                C.[Description],
                C.[Type],
                C.[Label]
                FROM [dbo].[CustomField] C (NOLOCK)
                JOIN [dbo].[EvaluationFormatCustomField] FC (NOLOCK)
                ON C.[VCC] = FC.[VCC] AND C.[ID] = FC.[CustomFieldId]
                JOIN [dbo].[EvaluationPlan] EP (NOLOCK) 
                ON EP.[Instance] = FC.[VCC] AND EP.[IdEvaluationFormat] = FC.[EvaluationFormatId]
                GROUP BY       
                C.[VCC],
                C.[ID],
                C.[Description],
                C.[Type],
                C.[Label]  

                INSERT INTO [dbo].[HistoricCustomFieldResult]
                SELECT
                CR.[VCC],
                CR.[ID],
                CR.[Description],
                CR.[Order],
                CR.[CustomFieldId]
                FROM [dbo].[CustomFieldResult] CR (NOLOCK)
                JOIN [dbo].[CustomField] C (NOLOCK)
                ON CR.[VCC] = C.[VCC] AND CR.[CustomFieldId] = C.[ID]
                JOIN [dbo].[EvaluationFormatCustomField] FC (NOLOCK)
                ON C.[VCC] = FC.[VCC] AND C.[ID] = FC.[CustomFieldId]
                JOIN [dbo].[EvaluationPlan] EP (NOLOCK) 
                ON EP.[Instance] = C.[VCC] AND EP.[IdEvaluationFormat] = FC.[EvaluationFormatId]
                GROUP BY 
                CR.[VCC],
                CR.[ID],
                CR.[Description],
                CR.[Order],
                CR.[CustomFieldId]


                INSERT INTO 
                [dbo].[HistoricEvaluationFormatAttachedSignature]
                SELECT 
                    AFS.[VCC],
                    AFS.[Description],
                    AFS.[Order],
                    AFS.[EvaluationFormatId]
                FROM [dbo].[EvaluationFormatAttachedSignature] AFS (NOLOCK)
                JOIN [dbo].[EvaluationPlan] EP (NOLOCK) 
                ON EP.[Instance] = AFS.[VCC] AND EP.[IdEvaluationFormat] = AFS.[EvaluationFormatId]
                GROUP BY 
                    AFS.[VCC],
                    AFS.[Description],
                    AFS.[Order],
                    AFS.[EvaluationFormatId]

                INSERT INTO [dbo].[HistoricCampaignEvaluationFormat]
                SELECT
                    CEF.[Instance],
                    CEF.[IdEvaluationFormat],
                    CEF.[IdCampaign]
                FROM [dbo].[CampaignEvaluationFormat] CEF (NOLOCK) 
                JOIN [dbo].[EvaluationPlan] EP (NOLOCK) 
                ON EP.[Instance] = CEF.[Instance] AND EP.[IdEvaluationFormat] = CEF.[IdEvaluationFormat]
                GROUP BY 
                    CEF.[Instance],
                    CEF.[IdEvaluationFormat],
                    CEF.[IdCampaign]



                    INSERT INTO [dbo].[HistoricTemplate] 
                    SELECT 
                        T.[VCC],
                        T.[Name],
                        T.[Type],
                        T.[Description],
                        T.[Template],
                        T.[Category]
                    FROM [dbo].[Template] T (NOLOCK)
                    JOIN [dbo].[Study] S (NOLOCK) 
                    ON S.[VCC] = T.[VCC] AND S.[TemplateId] = T.[Name]
                    GROUP BY 
                        T.[VCC],
                        T.[Name],
                        T.[Type],
                        T.[Description],
                        T.[Template],
                        T.[Category]

                    INSERT INTO [dbo].[HistoricTemplateTheme]
                    SELECT
                        TT.[VCC],
                        TT.[TemplateId],
                        TT.[ID],
                        TT.[Order],
                        TT.[Description],
                        TT.[Category1],
                        TT.[Category2],
                        TT.[AnalysisType],
                        TT.[IsNegative]
                    FROM [dbo].[TemplateTheme] TT (NOLOCK)
                    JOIN [dbo].[Template] T (NOLOCK)
                    ON TT.[VCC] = T.[VCC] AND TT.[TemplateId] = T.[Name]
                    JOIN [dbo].[Study] S (NOLOCK) 
                    ON S.[VCC] = T.[VCC] AND S.[TemplateId] = T.[Name]
                    GROUP BY
                        TT.[VCC],
                        TT.[TemplateId],
                        TT.[ID],
                        TT.[Order],
                        TT.[Description],
                        TT.[Category1],
                        TT.[Category2],
                        TT.[AnalysisType],
                        TT.[IsNegative]


                    INSERT INTO [dbo].[HistoricTemplateThemeItem]
                    SELECT 
                        TTI.[VCC],
                        TTI.[TemplateId],
                        TTI.[TemplateThemeId],
                        TTI.[ID],
                        TTI.[AcceptancePercentage]
                    FROM [dbo].[TemplateThemeItem] TTI (NOLOCK)
                    JOIN [dbo].[Template] T (NOLOCK)
                    ON TTI.[VCC] = T.[VCC] AND TTI.[TemplateId] = T.[Name]
                    JOIN [dbo].[Study] S (NOLOCK) 
                    ON S.[VCC] = T.[VCC] AND S.[TemplateId] = T.[Name]
                    GROUP BY 
                        TTI.[VCC],
                        TTI.[TemplateId],
                        TTI.[TemplateThemeId],
                        TTI.[ID],
                        TTI.[AcceptancePercentage]


                    INSERT INTO [dbo].[HistoricTemplateThemeItemEntity]
                    SELECT 
                        TTIE.[VCC],
                        TTIE.[TemplateId],
                        TTIE.[TemplateThemeId],
                        TTIE.[TemplateThemeItemId],
                        TTIE.[EntityId],
                        TTIE.[EntityName],
                        TTIE.[AcceptancePercentage],
                        TTIE.[Order]
                    FROM [dbo].[TemplateThemeItemEntity] TTIE (NOLOCK)
                    JOIN [dbo].[Study] S (NOLOCK) 
                    ON S.[VCC] = TTIE.[VCC] AND S.[TemplateId] = TTIE.[TemplateId]
                    GROUP BY 
                        TTIE.[VCC],
                        TTIE.[TemplateId],
                        TTIE.[TemplateThemeId],
                        TTIE.[TemplateThemeItemId],
                        TTIE.[EntityId],
                        TTIE.[EntityName],
                        TTIE.[AcceptancePercentage],
                        TTIE.[Order]


                INSERT INTO [dbo].[HistoricGenericTemplateQuery]
                SELECT
                    GTQ.[VCC],
                    GTQ.[TemplateId],
                    GTQ.[ID],
                    GTQ.[Query]
                FROM [dbo].[GenericTemplateQuery] GTQ (NOLOCK)
                JOIN [dbo].[Study] S (NOLOCK) 
                ON S.[VCC] = GTQ.[VCC] AND S.[TemplateId] = GTQ.[TemplateId]
                GROUP BY
                    GTQ.[VCC],
                    GTQ.[TemplateId],
                    GTQ.[ID],
                    GTQ.[Query]



    -- AQUI TERMINA LA SECCION CON LOS INSERT/UPDATE/DELETE PARA LA NUEVA VERSION



    

    -- TERMINAMOS LA ACTUALIZACION DE DATA ACTUALIZANDO EL BUILDVERSION
    UPDATE [dbo].[Version] 
        SET [BuildVersion] = @NEW_DATABASE_BUILD_VERSION, 
            [IsRegistered] = 0,
            [TimeStamp] = GETUTCDATE() 
        WHERE [BuildVersion] = @OLD_DATABASE_BUILD_VERSION
    PRINT 'UPDATED_DATABASE_BUILD_VERSION'

    -- COMPLETAMOS LA TRANSACCION
    COMMIT TRANSACTION
END TRY
BEGIN CATCH
    IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

    DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
    DECLARE @Severity integer = ERROR_SEVERITY()
    DECLARE @State integer = ERROR_STATE()

    RAISERROR(@Message, @Severity, @State)
    --RETURN -1
END CATCH



