PRINT N'Modificando [dbo].[DeleteProcess]...';


GO
ALTER PROCEDURE [dbo].[DeleteProcess] (
	@VCC Varchar(50) = NULL, @Name Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY

		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

			-- VERIFICAMOS SI EXISTE EL PROCESS A ELIMINAR
			IF EXISTS(
				SELECT [Name] 
					FROM [dbo].[Process] (NOLOCK)
					WHERE [VCC] = @VCC 
					AND [Name] = @Name
			)
			BEGIN
				-- ELIMINAMOS DE PROCESSSUMMARY
				DELETE FROM [dbo].[ProcessSummary]
					WHERE [VCC] = @VCC
					AND [ProcessId] = @Name

				-- ELIMINAMOS DE PROCESSAUDIO
				DELETE FROM [dbo].[ProcessAudio]
					WHERE [VCC] = @VCC
					AND [ProcessId] = @Name

				-- ELIMINAMOS DE PROCEESSFILTER
				DELETE FROM [dbo].[ProcessFilter]
					WHERE [VCC] = @VCC
					AND [ProcessId] = @Name

				-- ELIMINAMOS DE PROCESSAUDIOSUMMARY
				DELETE FROM [dbo].[ProcessAudioSummary]
					WHERE [VCC] = @VCC
					AND [ProcessId] = @Name

				-- ELIMINAMOS DE TRANSCRIPTION
				DELETE FROM [dbo].[Transcription]
					WHERE [VCC] = @VCC
					AND [ProcessId] = @Name

				-- ELIMINAMOS DE PROGRESS
				DELETE FROM [dbo].[Progress]
					WHERE [VCC] = @VCC
					AND [ProcessId] = @Name

				-- ELIMINAMOS LA DATA POSIBLE DATA DE UNA PLANTILLA AGENDADA (STUDYRESULTSUMMARYDETAIL)
				DELETE FROM [dbo].[StudyResultSummaryDetail] 
					WHERE [VCC] = @VCC 
						AND [StudyId] IN (
							SELECT S.[Name] 
								FROM [Study] S (NOLOCK) 
								WHERE S.[ProcessId] = @Name
									AND S.[Type] = 'TEMPLATE'
						)

				-- ELIMINAMOS LA DATA POSIBLE DATA DE UNA PLANTILLA AGENDADA (STUDYRESULTSUMMARY)
				DELETE FROM [dbo].[StudyResultSummary]
					WHERE [VCC] = @VCC 
						AND [StudyId] IN (
							SELECT S.[Name] 
								FROM [Study] S (NOLOCK) 
								WHERE S.[ProcessId] = @Name
									AND S.[Type] = 'TEMPLATE'
						)

				-- ELIMINAMOS LA DATA POSIBLE DATA DE UNA PLANTILLA AGENDADA (STUDY)
				DELETE FROM [Study]
					WHERE [ProcessId] = @Name
						AND [Type] = 'TEMPLATE'

				-- ACTUALIZAMOS LOS REGISTROS DE AGENDA
				UPDATE [dbo].[BatchScheduleExecution]
					SET [Status] = 'DISCARDED_BY_DELETED_PROCESS',
						[LastModifiedDate] = GETUTCDATE(),
						[TimeStamp] = GETUTCDATE()
					WHERE [VCC] = @VCC 
						AND [ProcessId] = @NAME

				-- FINALIZAMOS ELIMINANDO DE PROCSS
				DELETE FROM [dbo].[Process]
					WHERE [VCC] = @VCC
					AND [Name] = @Name

				SELECT 1 AS 'result'
			END
			ELSE
			BEGIN
				SELECT 0 AS 'result'
			END

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH
    
    SET NOCOUNT OFF
END
GO
PRINT N'Modificando [dbo].[GetEvaluations]...';


GO
ALTER PROCEDURE [dbo].[GetEvaluations] (
	@Instance Varchar(50) = NULL,
	@IdAuditor VARCHAR(MAX) = NULL,
	@IdAgent Varchar(MAX) = NULL,
   	@IdCampaign VARCHAR(MAX) = NULL,
	@IdEvaluationPlan VARCHAR(MAX) = NULL,
 	@EvaluationDateFrom VARCHAR(50) = '1980-01-01T00:00:00',
    @EvaluationDateTo VARCHAR(50) = '2050-01-01T23:59:59',
	@EvaluationHourFrom VARCHAR(50) = '00:00:00',
    @EvaluationHourTo VARCHAR(50) = '23:59:59',
	@InteractionMinDuration Int = 0,
    @InteractionMaxDuration Int = 86400,
    @InteractionOrigins Varchar(50) = NULL,
	@InteractionResults Varchar(MAX) = NULL,
	@EvaluationTypes Varchar(MAX) = NULL,
	@InteractionTags Varchar(MAX) = NULL,
	@InteractionFormats Varchar(50) = NULL,
	@InteractionCoached VARCHAR(50) = NULL,
	@InteractionIds     VARCHAR(MAX) = NULL,
	@EvaluationsLimit Int = 0,
	@ContactAddress Varchar(300) = NULL
)
AS
BEGIN
	SET NOCOUNT ON
	IF (@EvaluationHourFrom = '') SET @EvaluationHourFrom = '00:00:00';
	IF (@EvaluationHourTo = '') SET @EvaluationHourTo = '23:59:59';
	
	-- SI ORIGINS TIENE DOS VALORES, IGNORAMOS EL FILTRO
	IF (SELECT COUNT(*) FROM [dbo].[split](@InteractionOrigins, ',')) = 2 SET @InteractionOrigins = ''
	
	DECLARE @SQL_SELECT_1 VARCHAR(MAX) = ''
	DECLARE @SQL_FROM VARCHAR(MAX) = ''
	DECLARE @SQL_FILTER VARCHAR(MAX) = ''
	DECLARE @SQL_EXTRA VARCHAR(MAX) = ''
	DECLARE @FILTERS_OPTIONAL_PART VARCHAR(MAX) = ''
    DECLARE @FILTERS_EXCLUDING_PART VARCHAR(MAX) = ''
    DECLARE @FILTERS_FIXED_PART     VARCHAR(MAX) = ''

	-- EMULAMOS UNA NUEVA TABLA EN BASE A INTERACTIONEVALUATIONPLAN
	SET @SQL_SELECT_1 += '
		;WITH CTE AS (
			SELECT ER.[Instance],
					ER.[IdEvaluationPlan],
					ER.[IdInteraction], 
					ER.[IdAgent],
					IEP.[AgentName],
					RN = ROW_NUMBER() OVER(PARTITION BY IEP.[Instance], IEP.[IdEvaluationPlan], IEP.[IdInteraction] ORDER BY IEP.[CreatedDate] ASC)
				FROM [dbo].[EvaluationResult] ER (NOLOCK) 
				INNER JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
					ON EP.[Instance] = ER.[Instance]
						AND EP.[Id] =  ER.[IdEvaluationPlan]
				INNER JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK) 
					ON (AEP.[Instance] = ER.[Instance] 
						AND AEP.[IdAuditor] = ER.[IdAuditor] 
						AND AEP.[IdInteraction] = ER.[IdInteraction] 
						AND AEP.[InteractionType] = ER.[InteractionType] 
						AND AEP.[IdEvaluationPlan] = ER.[IdEvaluationPlan]) 
				INNER JOIN [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
					ON IEP.[Instance] = ER.[Instance] 
						AND IEP.[IdEvaluationPlan] = EP.[ID]
						AND IEP.[IdInteraction] = ER.[IdInteraction]
						AND IEP.[IdAgent] = ER.[IdAgent]
				WHERE ER.[Instance] = ''' + @Instance + '''
					AND ER.[EvaluationDate] BETWEEN ''' + @EvaluationDateFrom + '''AND ''' + @EvaluationDateTo + '''
					AND EP.[Status] = CASE EP.[Evaluation_Mode] WHEN ''A'' THEN 7 ELSE EP.[Status] END 
					AND AEP.[Completed] = 1
		),
		SUMMARIZED_CTE AS (
			SELECT [Instance],
					[IdEvaluationPlan],
					[IdInteraction], 
					[IdAgent],
					[AgentName]
				FROM CTE
				WHERE RN = 1
		) '

	SET @SQL_SELECT_1 += '
		SELECT '

	SET @SQL_SELECT_1 += '
				AEP.Completed						''completed'',
				AEP.[StatusDescription]             ''statusDescription'',
				A.[ErrorMessage]                    ''errorMessage'',
    			AEP.score							''qualification'',
    			AEP.IdAuditor						''idAuditor'',
    			U.FirstName + '' '' + U.LastName	''auditorName'',
    			MAX(ER.EvaluationDate)				''evaluationDate'',
    			AEP.IdEvaluationPlan				''idEvaluationPlan'',
				EP.Description						''evaluationPlan'',
				EP.[Evaluation_Mode]				''evaluationType'',
    			AEP.idAgent							''idAgent'',
    			C.[AgentName]						''agentName'',							-- NOMBRE DEL AGENTE
				AEP.idInteraction					''idInteraction'',
				AEP.InteractionDate					''interactionDate'',
    			AEP.InteractionType					''interactionType'',
    			AEP.IdCampaign						''idCampaign'',
				E.[Type]							''environmentType'',
				IIF (
					ERV.[InteractionId] IS NULL,
					CAST(0 AS BIT),
					CAST(1 AS BIT)
				)									''review'',
				COUNT(ERC.[Id])						''commitments'' '

	SET @SQL_FROM += '
			FROM EvaluationResult ER (NOLOCK) 
			INNER JOIN SUMMARIZED_CTE C (NOLOCK) 
				ON C.[Instance] = ER.[Instance]
					AND C.[IdEvaluationPlan] = ER.[IdEvaluationPlan]
					AND C.[IdInteraction] = ER.[IdInteraction] 
					AND C.[IdAgent] = ER.[IdAgent] 
			INNER JOIN [EvaluationPlan] EP (NOLOCK) 
				ON (EP.[Instance] = ER.[Instance] 
					AND EP.[ID] = ER.IdEvaluationPlan) 
			LEFT JOIN [AuditorEvaluationPlan] AEP (NOLOCK) 
				ON (AEP.[Instance] = ER.[Instance] 
					AND AEP.[IdAuditor] = ER.[IdAuditor] 
					AND AEP.[IdInteraction] = ER.[IdInteraction] 
					AND AEP.[InteractionType] = ER.[InteractionType] 
					AND AEP.[IdEvaluationPlan] = ER.[IdEvaluationPlan]) 
			LEFT JOIN [dbo].[EvaluationReview] ERV (NOLOCK)
				ON ERV.[VCC] = AEP.[Instance]
					AND ERV.[EvaluationPlanId] = AEP.[IdEvaluationPlan]
					AND ERV.[InteractionId] = AEP.[IdInteraction]
			LEFT JOIN [dbo].[EvaluationReviewCommitment] ERC (NOLOCK)
				ON ERC.[VCC] = AEP.[Instance] 
					AND ERC.[EvaluationPlanId] = AEP.[IdEvaluationPlan]
					AND ERC.[InteractionId] = AEP.[IdInteraction]
					-- AND ERC.[SupervisorId] = AEP.[IdAuditor] 
					AND ERC.[AgentId] = AEP.[idAgent]
			LEFT JOIN [dbo].[Audio] A (NOLOCK)
			    ON A.[VCC] = AEP.[Instance]
			        AND A.[ID] = AEP.[IdInteraction]
			LEFT JOIN [dbo].[Environment] E (NOLOCK)
				ON E.[VCC] = ER.[Instance]
					AND E.[Name] = EP.[EnvironmentId]
			LEFT JOIN [User] U (NOLOCK) 
				ON (U.[VCC] = ER.[Instance] 
					AND U.ID = AEP.IdAuditor) '

	SET @FILTERS_FIXED_PART = '
			WHERE ER.[Instance] = ''' + @Instance + '''
				AND ER.[EvaluationDate] >= ''' + @EvaluationDateFrom + '''
				AND ER.[EvaluationDate] <= ''' + @EvaluationDateTo + '''
				AND EP.[Status] = CASE EP.[Evaluation_Mode] WHEN ''A'' THEN 7 ELSE EP.[Status] END 
				AND AEP.[Completed] = 1 ' -- LINEA PARA FILTRAR EVALUACIONES COMPLETADAS

	IF (@IdAuditor IS NOT NULL AND @IdAuditor != '') SET @FILTERS_OPTIONAL_PART += 'AND AEP.[IdAuditor] IN (SELECT Value FROM [dbo].[split](''' + @IdAuditor + ''','','')) '
	IF (@IdAgent IS NOT NULL AND @IdAgent != '') SET @FILTERS_OPTIONAL_PART += 'AND AEP.[IdAgent] IN (SELECT Value FROM [dbo].[split](''' + @IdAgent + ''','','')) '
	IF (@IdCampaign IS NOT NULL AND @IdCampaign != '') SET @FILTERS_OPTIONAL_PART += 'AND AEP.[IdCampaign] IN (SELECT Value FROM [dbo].[split](''' + @IdCampaign + ''','','')) '
	IF (@IdEvaluationPlan IS NOT NULL AND @IdEvaluationPlan != '') SET @FILTERS_OPTIONAL_PART += 'AND EP.[ID] IN (SELECT Value FROM [dbo].[split](''' + @IdEvaluationPlan + ''','','')) '
	IF (CAST(@EvaluationHourFrom AS TIME) <= CAST(@EvaluationHourTo AS TIME)) SET @FILTERS_OPTIONAL_PART += 'AND CAST(ER.[EvaluationDate] AS TIME) BETWEEN ''' + @EvaluationHourFrom + ''' AND ''' + @EvaluationHourTo + ''' '
	IF (CAST(@EvaluationHourFrom AS TIME) > CAST(@EvaluationHourTo AS TIME)) SET @FILTERS_OPTIONAL_PART += 'AND CAST(ER.[EvaluationDate] AS TIME) NOT BETWEEN ''' + @EvaluationHourTo + ''' AND ''' + @EvaluationHourFrom + ''' '
	IF (@InteractionMinDuration IS NOT NULL) SET @FILTERS_OPTIONAL_PART += 'AND AEP.[InteractionDuration] >= ' + (SELECT CONVERT(varchar(10), @InteractionMinDuration)) + ' '
	IF (@InteractionMaxDuration IS NOT NULL) SET @FILTERS_OPTIONAL_PART += 'AND AEP.[InteractionDuration] <= ' + (SELECT CONVERT(varchar(10), @InteractionMaxDuration))+ ' '
	IF (@InteractionOrigins IS NOT NULL AND @InteractionOrigins != '') SET @FILTERS_OPTIONAL_PART += 'AND AEP.[InteractionOrigin] IN (SELECT Value FROM [dbo].[split](''' + @InteractionOrigins + ''','','')) '
	--IF (@InteractionResults IS NOT NULL AND @InteractionResults != '') SET @FILTERS_OPTIONAL_PART += 'AND CONCAT(AEP.[IdCampaign], ''_'', AEP.[ManagementResult]) IN (SELECT Value FROM [dbo].[split](''' + @InteractionResults + ''','','')) '
	IF (@InteractionResults IS NOT NULL AND @InteractionResults != '') 
		SET @FILTERS_OPTIONAL_PART += 'AND (
								(CONCAT(AEP.[IdCampaign], ISNULL(REPLACE(AEP.[ManagementResult], ''/'', ''_''), ''_'')) IN (SELECT Value FROM [dbo].[split](''' + @InteractionResults + ''','','')))
								OR
								(CONCAT(AEP.[IdCampaign], ''_'', ISNULL(AEP.[ManagementResult], ''_'')) IN (SELECT Value FROM [dbo].[split](''' + @InteractionResults + ''','','')))
								OR
								(AEP.[ManagementResult] IN (SELECT Value FROM [dbo].[split](''' + @InteractionResults + ''','','')))
							)'
	
	IF (@EvaluationTypes IS NOT NULL AND @EvaluationTypes != '') 
	BEGIN
		SET @FILTERS_OPTIONAL_PART += '
				AND EP.[Evaluation_Mode] IN (SELECT Value FROM [dbo].[split](''' + @EvaluationTypes + ''','',''))  '
	END

	IF(@ContactAddress <> '')
		BEGIN
			SET @FILTERS_OPTIONAL_PART += '
				AND AEP.[ContactAddress] LIKE ''%' + @ContactAddress + '%''
			'
	END	
	
	IF (@InteractionTags IS NOT NULL AND @InteractionTags != '') 
	BEGIN
		SET @SQL_FROM += '
			LEFT JOIN [Tag] T (NOLOCK) 
				ON (T.[VCC] = ER.[Instance] 
					AND T.[Tag] IN (SELECT Value FROM [dbo].[split](''' + @InteractionTags + ''','',''))) '
		SET @FILTERS_OPTIONAL_PART += '
				AND T.[AudioId] LIKE CONCAT(AEP.[IdInteraction], ''%'') 
				AND T.[AudioId] IS NOT NULL '
	END
	
	--IF (@InteractionFormats IS NOT NULL AND @InteractionFormats != '') SET @FILTERS_OPTIONAL_PART += 'AND AEP.[InteractionFormat] IN (SELECT Value FROM [dbo].[split](''' + @InteractionFormats + ''','','')) '

	IF (@InteractionIds IS NOT NULL AND @InteractionIds <> '')
    BEGIN
        SET @FILTERS_OPTIONAL_PART = ''
	    SET @FILTERS_EXCLUDING_PART += ' AND AEP.[IdInteraction] IN (SELECT Value FROM [dbo].[split](''' + @InteractionIds + ''','','')) '
	END

	SET @SQL_EXTRA += '
			GROUP BY AEP.Completed,
						AEP.[StatusDescription],
						A.[ErrorMessage],
						AEP.score,
						AEP.IdAuditor,
						U.FirstName + '' '' + U.LastName,
						AEP.IdEvaluationPlan,
						EP.Description,
						EP.[Evaluation_Mode],
						AEP.idAgent,
						C.[AgentName],
						AEP.idInteraction,
						AEP.InteractionDate,
						AEP.InteractionType,
						AEP.IdCampaign, 
						E.[Type],
						ERV.[InteractionId]'

	IF @InteractionCoached IS NOT NULL AND @InteractionCoached != ''
	BEGIN
		IF @InteractionCoached = 'true'
			SET @SQL_EXTRA += '
				HAVING COUNT(ERC.[Id]) >= 1 '
		ELSE 
			SET @SQL_EXTRA += '
				HAVING COUNT(ERC.[Id]) = 0 '
	END

	SET @SQL_EXTRA += '
			ORDER BY MAX(ER.EvaluationDate) DESC '

	SET @SQL_SELECT_1 += @SQL_FROM  + @FILTERS_FIXED_PART + @FILTERS_OPTIONAL_PART + @FILTERS_EXCLUDING_PART + @SQL_EXTRA 

	-- SELECT @SQL_SELECT_1
	EXEC (@SQL_SELECT_1)
	
END
GO
PRINT N'Modificando [dbo].[GetProcessUsage]...';


GO
ALTER PROCEDURE [dbo].[GetProcessUsage]
(
	@VCC				VARCHAR(50),
	@ProcessId			VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON

	BEGIN
		-- Validar para Proceso. Avisar si esta en progreso.
		SELECT [Name]				'name'
			FROM [dbo].[Process] (NOLOCK)
			WHERE [VCC] = @VCC
				AND [Name] = @ProcessId
				AND [Status] NOT IN('COMPLETE', 'FAILED')
	END

	-- LISTAMOS LOS ESTUDIOS REGULARES
	BEGIN
		-- Validar para Estudio. Avisar si esta asociado a un estudio.
		SELECT DISTINCT [Name]		'name'
			FROM [dbo].[Study] (NOLOCK)
			WHERE [VCC] = @VCC
				AND [ProcessId] = @ProcessId
				AND [Type] = 'SPEECH'
	END

	-- LISTAMOS LOS ESTUDIOS DE PLANTILLA AGENDADA
	BEGIN
		SELECT DISTINCT [Name]		'name'
			FROM [dbo].[Study] (NOLOCK)
			WHERE [VCC] = @VCC
				AND [ProcessId] = @ProcessId
				AND [Type] = 'TEMPLATE'
	END

	SET NOCOUNT OFF
END
GO
PRINT N'Actualización completada.';


GO
